/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <asm/mmap.h>

	.text
	.global _prolog

_prolog:

	nextpc r2	/* _prolog = r2 - 4 */

/* word aligned */

_rs_init:

	/*
	   The Nios II caches cannot be disabled by software;
	   they are always enabled.
	*/

	/* ICACHE INIT */

	/*
	   It is the responsibility of the first eight instructions
	   of the reset handler to initialize the remainder of the
	   instruction cache.
	*/

	/* xor r0, r0, r0 */	/* nios2 ABI / r0 is always 0 */

	mov   r4, r0
	movhi r5, %hi(CPU_ICACHE_SIZE)
	ori   r5, r5, %lo(CPU_ICACHE_SIZE)
_0:
	initi r4
	addi  r4, r4, CPU_ICACHELINE_SIZE
	bltu  r4, r5, _0

	/* DCACHE INIT */

	mov   r4, r0
	movhi r5, %hi(CPU_DCACHE_SIZE)
	ori   r5, r5, %lo(CPU_DCACHE_SIZE)
_1:
	initd 0(r4)
	addi  r4, r4, CPU_DCACHELINE_SIZE
	bltu  r4, r5, _1

	/* disable exceptions & interrupts */

	wrctl status, r0	/* global */
	wrctl ienable, r0	/* all vectors */

	/* RELOCATE CODE & DATA (.text + .data) */

/*
	.text	_prolog     => _etext
	.data	_data       => _edata
	.bssi   __bss_start => _end
*/

	addi  r4, r2, -4
	movhi r5, %hi(_edata)
	ori   r5, r5, %lo(_edata)
	movhi r6, %hi(_prolog)
	ori   r6, r6, %lo(_prolog)
	sub   r5, r5, r6
	add   r5, r4, r5
_2:	ldwio r7, 0(r4)
	stwio r7, 0(r6)
	addi  r6, r6, 4
	cmpeq r2, r4, r5
	addi  r4, r4, 4
	beq   r2, r0, _2

	/* JUMP TO RELOC ADDR */

	movhi r4, %hi(_entry)
	ori   r4, r4, %lo(_entry)
	callr r4

	.global _entry

_entry:

	/* binary load in ram at exec location / no need for relocation now */

	/* COPY EXCEPTIONS ENTRY ROUTINE */

	movhi r4, %hi(_ex_prolog)
	ori   r4, r4, %lo(_ex_prolog)	
	movhi r5, %hi(_ex_end)
	ori   r5, r5, %lo(_ex_end)
	movhi r6, %hi(SYS_EX_VECTOR)
	ori   r6, r6, %lo(SYS_EX_VECTOR)
_3:	ldwio r7, 0(r4)
	stwio r7, 0(r6)
	addi  r6, r6, 4
	cmpeq r2, r4, r5
	addi  r4, r4, 4
	beq   r2, r0, _3

	/* ZERO BSS/SBSS -- bss and sbss are assumed to be adjacent
	 * and between __bss_start and _end.
	 */

	movhi r4, %hi(__bss_start)
	ori   r4, r4, %lo(__bss_start)	
	movhi r5, %hi(_end)
	ori   r5, r5, %lo(_end)
	cmpeq r6, r4, r5
	bne   r6, r0, _5
_4:	stwio r0, 0(r4)
	addi  r4, r4, 4
	cmpeq r6, r4, r5
	beq   r6, r0, _4
_5:

	/* STACK INIT */

	movhi sp, %hi(SYS_INIT_SP)
	ori   sp, sp, %lo(SYS_INIT_SP)

	/* flag gp (unsed see -G0 cc flag) */

	movhi gp, %hi(0xdeadbeef)
	ori   gp, gp, %lo(0xdeadbeef)

	/*
	 * Call tiny -- never returns
	 */

	br tiny

	/* 
	 * branch to the _prolog just in case ;-)
	 */

	br _prolog

	.globl	_version

_version:
	.ascii "TINY"
	.ascii " (", __DATE__, " - ", __TIME__, ")", "\n\0"

