/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <asm/mmap.h>
#include <serial.h>
#include <channel.h>
#include <sysutil.h>
#include <vt100.h>

#include <fb.h>
#include <fbcon.h>

#include <malloc.h>

#include "../data/sprite_mem_00_32.h"
#include "../data/sprite_mem_01_32.h"
#include "../data/sprite_mem_02_32.h"
#include "../data/sprite_mem_03_32.h"
#include "../data/sprite_mem_04_32.h"
#include "../data/sprite_mem_05_32.h"

#include <nios2.h>

#include <ei.h>
#include <irq.h>

struct epcs_s{
   unsigned int sector_cnt;
   unsigned int sector_size;             
   unsigned int page_size;
};

unsigned char malloc_area[2*1024*1024] __attribute__ ((aligned(4)));;

unsigned short fb_0[480*272] __attribute__ ((aligned(0x100)));;

/*
unsigned short sprite_mem_00_16[48*48] __attribute__ ((aligned(2)));;
unsigned short sprite_mem_01_16[48*48] __attribute__ ((aligned(2)));;
unsigned short sprite_mem_02_16[48*48] __attribute__ ((aligned(2)));;
*/
unsigned short sprite_mem_03_16[96*96] __attribute__ ((aligned(2)));;
unsigned short sprite_mem_04_16[64*64] __attribute__ ((aligned(2)));;
unsigned short sprite_mem_05_16[200*100] __attribute__ ((aligned(2)));;

static volatile unsigned int timeout = 0;

unsigned short rgb24to16(unsigned char r, unsigned char g, unsigned char b)
{
   unsigned short color_16;
   unsigned int color_32;

   color_32 = (unsigned int)0x00 | b;
   color_32 |= g << 8;
   color_32 |= r << 16;

   color_16 = (unsigned short)0x00 | ((color_32 & 0xf8) >> 3);
   color_16 |= ((color_32 >> 8) & 0xfc) << 3;
   color_16 |= ((color_32 >> 16) & 0xf8) << 8;

   return color_16;
}

void timer_irq_hdlr(void)
{
   timeout = 1;
   alt_timer_irqclr(IO_TIMER_00_BASE);
}

typedef struct fb_s{
	unsigned int width;
	unsigned int height;
	unsigned int bpp;
	unsigned char *base;
} fb_t;

typedef struct sprite_s{
	unsigned int width;
	unsigned int height;
	unsigned int bpp;
	unsigned char *base;
} sprite_t;

int sprite_blit_16(fb_t* fb, sprite_t* sprite, int x, int y)
{
	int i,j;
	unsigned char* sc;
	unsigned char* ds;
	
	sc = (unsigned char*)sprite->base;
	ds = (unsigned char*)fb->base+((x+(y*fb->width))<<1);

	for(i=0; i<sprite->height; i++)
	{
		for(j=0; j<(sprite->width)<<1; j++)
		{
			*ds++ = *sc++;
		}
		ds+=(fb->width-sprite->width)<<1;
	}

   return 0;
}

void sprite_cvrt_32_16(sprite_t* sprite_sc, sprite_t* sprite_ds)
{
	unsigned int i;
	unsigned short pxl;
	unsigned short* pxl_16;
	unsigned int* pxl_32;

	pxl_32 = (unsigned int*)sprite_sc->base;
	pxl_16 = (unsigned short*)sprite_ds->base;

   i = (sprite_sc->width)*(sprite_sc->height);
   while(i--)
   {
         pxl = ((*pxl_32) & 0xf8) >> 3;
         pxl |= ((*pxl_32 >> 8) & 0xfc) << 3;
         pxl |= ((*pxl_32 >> 16) & 0xf8) << 8;
         pxl_32++;

         *(pxl_16++) = pxl;
   }
}

void tiny(void)
{

	fb_t fb;
	sprite_t sprite_00_16;
	sprite_t sprite_01_16;
	sprite_t sprite_02_16;
	sprite_t sprite_03_16;
	sprite_t sprite_04_16;
	sprite_t sprite_05_16;
	sprite_t sprite_00_32;
	sprite_t sprite_01_32;
	sprite_t sprite_02_32;
	sprite_t sprite_03_32;
	sprite_t sprite_04_32;
	sprite_t sprite_05_32;

   unsigned char* sprite_mem_00_16;
   unsigned char* sprite_mem_01_16;
   unsigned char* sprite_mem_02_16;

/*
   channel_register(STDIN, (channel_handler_t*)uart_getc, \
               (void*)IO_DFLT_UART | 0x80000000);
   channel_register(STDOUT, (channel_handler_t*)uart_putc, \
               (void*)(IO_DFLT_UART | 0x80000000));
   channel_register(STDERR, (channel_handler_t*)uart_putc, \
               (void*)(IO_DFLT_UART | 0x80000000));
*/

   irq_init();

   fb_init(fb_0);
   fbcon_init((unsigned int)fb_0, 480, 272);

   channel_register(STDOUT, (channel_handler_t*)fbputc, \
               (void*)(0x00));
   channel_register(STDERR, (channel_handler_t*)fbputc, \
               (void*)(0x00));

   fb_l1_di();

   alt_timer_init(IO_TIMER_00_BASE | 0x80000000, SYS_CLK_FREQ, 100);
   alt_timer_start(IO_TIMER_00_BASE | 0x80000000);

   irq_register(IO_TIMER_00_IRQ, (irq_handler_t*)timer_irq_hdlr, 0x00);

   /* ... */

   int i;
   unsigned short pixel_16;

   pixel_16 = rgb24to16(0xff, 0x5d, 0x0d);

   for(i=0; i<480*272; i++)
      fb_0[i] = pixel_16;

   /* ... */

   printf("tiny nios2 [experimental_000]\n\n");

   malloc_init(malloc_area, sizeof(malloc_area), 128);
   malloc_stat();

   sprite_mem_00_16 = (unsigned char*)malloc(48*48*2);
   /* malloc_stat(); */
   sprite_mem_01_16 = (unsigned char*)malloc(48*48*2);
   /* malloc_stat(); */
   sprite_mem_02_16 = (unsigned char*)malloc(48*48*2);
   malloc_stat();

   putchar('\n');

   /* ... */

	fb.width = 480;
	fb.height = 272;
	fb.bpp = 16;
	fb.base = (unsigned char*)fb_0;

	sprite_00_32.width = 48;
	sprite_00_32.height = 48;
	sprite_00_32.bpp = 32;
	sprite_00_32.base = (unsigned char*)(sprite_mem_00_32+20);

	sprite_01_32.width = 48;
	sprite_01_32.height = 48;
	sprite_01_32.bpp = 32;
	sprite_01_32.base = (unsigned char*)(sprite_mem_01_32+20);

	sprite_02_32.width = 48;
	sprite_02_32.height = 48;
	sprite_02_32.bpp = 32;
	sprite_02_32.base = (unsigned char*)(sprite_mem_02_32+20);

	sprite_03_32.width = 96;
	sprite_03_32.height = 96;
	sprite_03_32.bpp = 32;
	sprite_03_32.base = (unsigned char*)(sprite_mem_03_32+20);

	sprite_04_32.width = 64;
	sprite_04_32.height = 64;
	sprite_04_32.bpp = 32;
	sprite_04_32.base = (unsigned char*)(sprite_mem_04_32+20);

	sprite_05_32.width = 200;
	sprite_05_32.height = 100;
	sprite_05_32.bpp = 32;
	sprite_05_32.base = (unsigned char*)(sprite_mem_05_32+20);

	sprite_00_16.width = 48;
	sprite_00_16.height = 48;
	sprite_00_16.bpp = 16;
	sprite_00_16.base = (unsigned char*)sprite_mem_00_16;

	sprite_01_16.width = 48;
	sprite_01_16.height = 48;
	sprite_01_16.bpp = 16;
	sprite_01_16.base = (unsigned char*)sprite_mem_01_16;

	sprite_02_16.width = 48;
	sprite_02_16.height = 48;
	sprite_02_16.bpp = 16;
	sprite_02_16.base = (unsigned char*)sprite_mem_02_16;

	sprite_03_16.width = 96;
	sprite_03_16.height = 96;
	sprite_03_16.bpp = 16;
	sprite_03_16.base = (unsigned char*)sprite_mem_03_16;

	sprite_04_16.width = 64;
	sprite_04_16.height = 64;
	sprite_04_16.bpp = 16;
	sprite_04_16.base = (unsigned char*)sprite_mem_04_16;

	sprite_05_16.width = 200;
	sprite_05_16.height = 100;
	sprite_05_16.bpp = 16;
	sprite_05_16.base = (unsigned char*)sprite_mem_05_16;

   sprite_cvrt_32_16(&sprite_00_32, &sprite_00_16);
   sprite_cvrt_32_16(&sprite_01_32, &sprite_01_16);
   sprite_cvrt_32_16(&sprite_02_32, &sprite_02_16);
   sprite_cvrt_32_16(&sprite_03_32, &sprite_03_16);
   sprite_cvrt_32_16(&sprite_04_32, &sprite_04_16);
   sprite_cvrt_32_16(&sprite_05_32, &sprite_05_16);

   // sprite_blit_16(&fb, &sprite_03_16, 348, 148);
   // sprite_blit_16(&fb, &sprite_04_16, 292, 28);
   // sprite_blit_16(&fb, &sprite_05_16, 52, 132);

   irq_enable();

   fb_l1_en();

   for(;;)
   {
      sprite_blit_16(&fb, &sprite_00_16, 416, 16);
      while(!timeout); timeout = 0;
      sprite_blit_16(&fb, &sprite_01_16, 416, 16);
      while(!timeout); timeout = 0;
      sprite_blit_16(&fb, &sprite_02_16, 416, 16);
      while(!timeout); timeout = 0;
   }

   /* ... */
}

