/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <types.h>
#include "i2c/i2c-gpio.h"
#include "24cxx.h"

static void mem_24cxx_ack_polling(unsigned char i2c_addr)
{
   int i=1;

   /* add a max_loop_count to prevent from infinite loop */

   while(i)
   {
      i2c_start();
      i=i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   }
   i2c_stop();
}

void mem_24c21_bidir_mode_en(void)
{

/*
 * Before the 24LC21A can be switched into the
 * Bidirectional mode (Figure 3-1), it must enter the
 * Transition mode, which is done by applying a valid
 * high-to-low transition on the Bidirectional mode Clock
 * (SCL).
*/
   
   i2c_start();
   i2c_stop();
}

int mem_24cxx_random_read(unsigned char i2c_addr, unsigned int wrd_addr)
{
   int i=0, val;

   i2c_start();  
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   i |= i2c_write_byte(wrd_addr);
   i2c_start();
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_RD);
   val = i2c_read_byte(I2C_NOACK);
   i2c_stop();

   if(i) return i;
   
   return val;
}

int mem_24cxx_byte_write(unsigned char val, unsigned char i2c_addr, unsigned int wrd_addr)
{
   int i=0;

   mem_24cxx_ack_polling(i2c_addr);    /* device busy ? */

   i2c_start();  
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   i |= i2c_write_byte(wrd_addr);
   i |= i2c_write_byte(val);
   i2c_stop();
   
   return i;
}
   
int mem_24cxx_sequential_read(unsigned char* buf, unsigned char i2c_addr, unsigned int offset, size_t length)
{
   int i=0;

   mem_24cxx_ack_polling(i2c_addr);    /* device busy ? */

   /* set the internal 24cxx counter to address 'offset' */

   i2c_start();    
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   i |= i2c_write_byte(offset);
   i2c_stop(); 

   /* sequential read */

   length--;

   i2c_start();
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_RD);
   while(length--)
      *(buf++) = (unsigned char)i2c_read_byte(I2C_ACK);
   *(buf++) = (unsigned char)i2c_read_byte(I2C_NOACK);
   i2c_stop();

   return i;
}

int mem_24cxx_page_write(unsigned char* buf, unsigned char i2c_addr, unsigned int offset, size_t length)
{
   int i=0;

   mem_24cxx_ack_polling(i2c_addr);    /* device busy ? */

   /* set the internal 24cxx counter to address 'offset' */

   i2c_start();    
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   i |= i2c_write_byte(offset);
   i2c_stop();

   /* perform page write */
   
   i2c_start();
   i |= i2c_set_ctrl(i2c_addr, I2C_CTRL_WR);
   while(length--)
      i |=  i2c_write_byte(*buf++);
   i2c_stop();

   return i;
}

