/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <asm/mmap.h>
#include <string.h>
#include "fb.h"

/* static struct fb_s fb; */

#define DISPLAY_HRES   480
#define DISPLAY_VRES   272

#define IO_MM_GFX_BASE 0x02000800

void fb_l1_en(void)
{
   lcd_ctrl_t* gfx;
   
   gfx = (lcd_ctrl_t*)(IO_MM_GFX_BASE | 0x80000000);

   gfx->status = 0x01;  /* clear interrupt */
   
   /* compiler optimization (-O2 / -O3) void this statement */
   while(!(gfx->status & 0x01));

   gfx->ctrl &= ~(unsigned int)0x06;  /* l1 & dma1 disabled */
   gfx->ctrl |= (unsigned int)0x02;   /* dma1 enable */
   gfx->ctrl |= (unsigned int)0x04;   /* l1 enable */
}

void fb_l1_di(void)
{
   lcd_ctrl_t* gfx;
   
   gfx = (lcd_ctrl_t*)(IO_MM_GFX_BASE | 0x80000000);

   gfx->status = 0x01;  /* clear interrupt */

   /* compiler optimization (-O2 / -O3) void this statement */
   while(!(gfx->status & 0x01));

   gfx->ctrl &= ~(unsigned int)0x06;  /* l1 & dma1 disabled */
}

void fb_l1_swap_buffer(unsigned int fb_base)
{
   lcd_ctrl_t* gfx;
   
   gfx = (lcd_ctrl_t*)(IO_MM_GFX_BASE | 0x80000000);

   fb_l1_di();

   gfx->l1_dma_start = (unsigned int)fb_base;

   fb_l1_en();
}

/* 480x272 */
int fb_init(unsigned short* fb_base)
{

   lcd_ctrl_t* gfx;

   gfx = (lcd_ctrl_t*)(IO_MM_GFX_BASE | 0x80000000);

   /* warn reset or sw download => ip already running */

   gfx->ctrl |= 0x01;   /* enable interrupt */

   fb_l1_di();

   gfx->ctrl &= ~0x01;  /* disable interrupt */
   
   /* ... */

   gfx->l1_dma_start = (unsigned int)fb_base;
   gfx->l1_dma_len = (unsigned int)(DISPLAY_HRES*DISPLAY_VRES);

   memset(fb_base, 0x00, DISPLAY_HRES*DISPLAY_VRES*2);

   /* irq */

   gfx->status = 0x01;     /* clear interrupt */
   gfx->ctrl |= 0x01;   /* enable interrupt */

   /* enable layer 1 */

   gfx->status = 0x01;  /* clear interrupt */

   /* compiler optimization (-O2 / -O3) void this statement */
   while(!(gfx->status & 0x01));

   gfx->ctrl &= ~(unsigned int)0x06;  /* l1 & dma1 disabled */
   gfx->ctrl |= (unsigned int)0x02;   /* dma1 enable */
   gfx->ctrl |= (unsigned int)0x04;   /* l1 enable */

   return 0;
}

