/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include "gpio/gpio.h"
#include "i2c-gpio.h"

static i2c_gpio_channel_data_t i2c_channel_data = {
   .gpio_base = 0xffffffff,
   .scl_pin = 0,
   .sda_pin = 0,
   .udelay = 0,
   .timeout = 0
};

/* ... */
static void __wait(int count)
{
   while((volatile int)count--);
}
   
static int i2c_gpio_setsda_val(i2c_gpio_channel_data_t* i2c_d, int value)
{
   return gpio_set_value(i2c_d->sda_pin, value, i2c_d->gpio_base);
}

static int i2c_gpio_setscl_val(i2c_gpio_channel_data_t* i2c_d, int value)
{
   return gpio_set_value(i2c_d->scl_pin, value, i2c_d->gpio_base);
}

static int i2c_gpio_setsda_dir(i2c_gpio_channel_data_t* i2c_d, int dir)
{
   if (dir)
      return gpio_direction_output(i2c_d->sda_pin, i2c_d->gpio_base);   
   else
      return gpio_direction_input(i2c_d->sda_pin, i2c_d->gpio_base);
}

static int i2c_gpio_setscl_dir(i2c_gpio_channel_data_t* i2c_d, int dir)
{
   if (dir)
      return gpio_direction_output(i2c_d->scl_pin, i2c_d->gpio_base);
   else
      return gpio_direction_input(i2c_d->scl_pin, i2c_d->gpio_base);
}

static int i2c_gpio_getsda(i2c_gpio_channel_data_t* i2c_d)
{
   return gpio_get_value(i2c_d->sda_pin, i2c_d->gpio_base);
}

static int i2c_gpio_getscl(i2c_gpio_channel_data_t* i2c_d)
{
   return gpio_get_value(i2c_d->scl_pin, i2c_d->gpio_base);
}

void i2c_start(void)
{
   i2c_gpio_setsda_val(&i2c_channel_data, 1);
   i2c_gpio_setscl_val(&i2c_channel_data, 1);
   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   __wait(5000);
   i2c_gpio_setsda_val(&i2c_channel_data, 0);
   __wait(5000);
   i2c_gpio_setscl_val(&i2c_channel_data, 0);
   __wait(5000);
}

void i2c_stop(void)
{
   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   __wait(5000);
   i2c_gpio_setsda_val(&i2c_channel_data, 0);
   __wait(5000);
   i2c_gpio_setscl_val(&i2c_channel_data, 0);
   __wait(5000);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_IN);
   __wait(5000);
   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_IN);
   __wait(5000);
}

int i2c_write_byte(int value)
{
   int i;
   unsigned int timeout;
   
   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   
   /* send 8bits */

   for(i=7; i>=0; i--)
   {
      if((value >> i) & 0x0001)
      {
         i2c_gpio_setsda_val(&i2c_channel_data, 1);
         __wait(5000);
      }else{
         i2c_gpio_setsda_val(&i2c_channel_data, 0);
         __wait(5000);
      }

      i2c_gpio_setscl_val(&i2c_channel_data, 1);
      __wait(5000);
      i2c_gpio_setscl_val(&i2c_channel_data, 0);
      __wait(5000);
   }

/*
   sda ---|----hi-z-----|--- (master side)
   
   sda ___            __  _
          \          /  \/
       ___/\________/    \_
   
   scl ___        __
          \      /  \
           \____/    \_____
*/

   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_IN);
      i2c_gpio_setscl_val(&i2c_channel_data, 1);
   __wait(5000);

   timeout = 50000;
   while((volatile int)timeout--)
      if (!(i2c_gpio_getsda(&i2c_channel_data))) break;

   if(++timeout)
   {
      i2c_gpio_setscl_val(&i2c_channel_data, 0);
      __wait(5000);
/*
      while(!i2c_gpio_getsda(&i2c_channel_data));
     i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
*/
     __wait(5000);
      return 0;
   }else{
      i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_IN);
      i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_IN);
      return -1;  /* timeout: no ack received */
   }
}


int i2c_read_byte(int ack)
{
   int i;
   unsigned int val = 0;

   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_IN);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_OUT);
   __wait(5000);

   /* send 8bits */

   for(i=0; i<8; i++)
   {
      i2c_gpio_setscl_val(&i2c_channel_data, 1);
      __wait(5000);

      val |= i2c_gpio_getsda(&i2c_channel_data);
      val = val << 1;

      i2c_gpio_setscl_val(&i2c_channel_data, 0);
      __wait(5000);
   }

   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_OUT);

   if(ack){    /* ack */
      i2c_gpio_setsda_val(&i2c_channel_data, 0);
   }else{      /* no ack*/
      i2c_gpio_setsda_val(&i2c_channel_data, 1);
   }
   __wait(5000);

   i2c_gpio_setscl_val(&i2c_channel_data, 1);
   __wait(5000);
   i2c_gpio_setscl_val(&i2c_channel_data, 0);

   return val >> 1;
}

int i2c_set_ctrl(int value, int mode)
{
   if(mode){
      value |= (int)0x0001;
   }else{
      value &= ~(int)0x0001;
   }
   
   return i2c_write_byte(value);
}

void i2c_set_channel(i2c_gpio_channel_data_t* i2c_d)
{
   i2c_channel_data.gpio_base = i2c_d -> gpio_base;
   i2c_channel_data.scl_pin = i2c_d -> scl_pin;
   i2c_channel_data.sda_pin = i2c_d -> sda_pin;
   i2c_channel_data.udelay = i2c_d -> udelay;
   i2c_channel_data.timeout = i2c_d -> timeout;

   i2c_gpio_setsda_dir(&i2c_channel_data, I2C_IO_DIR_IN);
   i2c_gpio_setscl_dir(&i2c_channel_data, I2C_IO_DIR_IN);
}

