/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <asm/nios2-io.h>
#include <asm/mmap.h>

/* base => 0xXXXXXXXX / tmrclk => hz / period => ms */

void alt_timer_init(int base, int tmrclk, int period)
{
   alt_timer_t* timer;

   timer = (alt_timer_t*)base;

   timer->control = ALT_TIMER_ITO | ALT_TIMER_CONT | ALT_TIMER_STOP;

   timer->periodl = ((tmrclk/1000)*(period-1)) & 0xffff;
   timer->periodh = (((tmrclk/1000)*(period-1)) >> 16) & 0xffff;
}

void alt_timer_start(int base)
{
   alt_timer_t* timer;
   unsigned int var;

   timer = (alt_timer_t*)base;

   /* warning start & stop bits return 1 after a read op */

   var = (timer->control & 0x00000003) | ALT_TIMER_START;
   timer->control = var;
}

void alt_timer_stop(int base)
{
   alt_timer_t* timer;
   unsigned int var;

   timer = (alt_timer_t*)base;

   /* warning start & stop bits return 1 after a read op */

   var = (timer->control & 0x00000003) | ALT_TIMER_STOP;
   timer->control = var;
}

inline void alt_timer_irqclr(int base)
{
   alt_timer_t* timer;

   timer = (alt_timer_t*)base;

   timer->status = 0;
}

