/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#ifndef __NIOS2_IO_H__
#define __NIOS2_IO_H__

/* TIMER */

typedef volatile struct alt_timer_s {
   unsigned status;     /* timer status register */
   unsigned control;    /* timer control register */
   unsigned periodl;    /* timeout period low */
   unsigned periodh;    /* timeout period high */
   unsigned snapl;      /* snapshot low */
   unsigned snaph;      /* snapshot high */
}alt_timer_t;

/* status register */
#define ALT_TIMER_TO    0x0001   /* timeout */
#define ALT_TIMER_RUN   0x0002   /* timer running */

/* control register */
#define ALT_TIMER_ITO   0x0001   /* timeout interrupt enable */
#define ALT_TIMER_CONT  0x0002   /* continuous mode */
#define ALT_TIMER_START 0x0004   /* start timer */
#define ALT_TIMER_STOP  0x0008   /* stop timer */

/* UART */

typedef volatile struct alt_uart_s {
   unsigned int rxdata;
   unsigned int txdata;
   unsigned int status;
   unsigned int control;
   unsigned int divisor;
   unsigned int endofpacket;
}alt_uart_t;

/* status register */
#define ALT_UART_PE     0x0001  /* parity error */
#define ALT_UART_FE     0x0002  /* frame error */
#define ALT_UART_BRK    0x0004  /* break detect */
#define ALT_UART_ROE    0x0008  /* rx overrun */
#define ALT_UART_TOE    0x0010  /* tx overrun */
#define ALT_UART_TMT    0x0020  /* tx empty */
#define ALT_UART_TRDY   0x0040  /* tx ready */
#define ALT_UART_RRDY   0x0080  /* rx ready */
#define ALT_UART_E      0x0100  /* exception */
/* 0x0200 not assigned */
#define ALT_UART_DCTS   0x0400  /* cts change */
#define ALT_UART_CTS    0x0800  /* cts */
#define ALT_UART_EOP    0x1000  /* eop detected */

/* control register */
#define ALT_UART_IPE    0x0001   /* parity error irq enable */
#define ALT_UART_IFE    0x0002   /* frame error irq enable */
#define ALT_UART_IBRK   0x0004   /* break detect irq enable */
#define ALT_UART_IROE   0x0008   /* rx overrun irq enable */
#define ALT_UART_ITOE   0x0010   /* tx overrun irq enable */
#define ALT_UART_ITMT   0x0020   /* tx empty irq enable */
#define ALT_UART_ITRDY  0x0040   /* tx ready irq enable */
#define ALT_UART_IRRDY  0x0080   /* rx ready irq enable */
#define ALT_UART_IE     0x0100   /* exception irq enable */
#define ALT_UART_TBRK   0x0200   /* transmit break */
#define ALT_UART_IDCTS  0x0400   /* cts change irq enable */
#define ALT_UART_RTS    0x0800   /* rts */
#define ALT_UART_IEOP   0x1000   /* eop detected irq enable */

/* JTAG UART */

typedef volatile struct alt_jtag_uart_s {
   unsigned int  data;
   unsigned int  control;
}alt_jtag_uart_t;

#define ALT_JTAG_UART_RAVAIL   0xffff0000
#define ALT_JTAG_UART_WSPACE   0xffff0000
#define ALT_JTAG_UART_RVALID   0x00008000
#define ALT_JTAG_UART_DATA     0x000000ff

/* PIO */

typedef volatile struct alt_pio_s {
   unsigned int data;            /* (read) data value currently on PIO inputs */
                                 /* (write) new value to drive on PIO outputs */
   unsigned int direction;       /* individual direction control */
   unsigned int interruptmask;   /* IRQ enable/disable (input ports) */
   unsigned int edgecapture;     /* edge detection */
   unsigned int outset;          /* set output port */
   unsigned int outclear;        /* clear output port */
}alt_pio_t;

#endif   /* __NIOS2_IO_H__ */
 
